@@ -39,19 +39,17 @@
 #include "vq.h"
 #include "arch.h"
 #include "os_support.h"
+#include "bands.h"
 #include "rate.h"
 
 #ifndef M_PI
 #define M_PI 3.141592653
 #endif
 
-
-static void exp_rotation1(celt_norm *X, int len, int dir, int stride, celt_word16 c, celt_word16 s)
+static void exp_rotation1(celt_norm *X, int len, int stride, celt_word16 c, celt_word16 s)
 {
    int i;
    celt_norm *Xptr;
-   if (dir>0)
-      s = -s;
    Xptr = X;
    for (i=0;i<len-stride;i++)
    {
@@ -72,12 +70,14 @@ static void exp_rotation1(celt_norm *X, int len, int dir, int stride, celt_word1
    }
 }
 
-static void exp_rotation(celt_norm *X, int len, int dir, int stride, int K)
+static void exp_rotation(celt_norm *X, int len, int dir, int stride, int K, int spread)
 {
+   static const int SPREAD_FACTOR[3]={5,10,15};
    int i;
    celt_word16 c, s;
    celt_word16 gain, theta;
    int stride2=0;
+   int factor;
    /*int i;
    if (len>=30)
    {
@@ -86,9 +86,11 @@ static void exp_rotation(celt_norm *X, int len, int dir, int stride, int K)
       X[14] = 1;
       K=5;
    }*/
-   if (2*K>=len)
+   if (2*K>=len || spread==SPREAD_NONE)
       return;
-   gain = celt_div((celt_word32)MULT16_16(Q15_ONE,len),(celt_word32)(len+10*K));
+   factor = SPREAD_FACTOR[spread-1];
+
+   gain = celt_div((celt_word32)MULT16_16(Q15_ONE,len),(celt_word32)(len+factor*K));
    /* FIXME: Make that HALF16 instead of HALF32 */
    theta = HALF32(MULT16_16_Q15(gain,gain));
 
@@ -110,12 +112,12 @@ static void exp_rotation(celt_norm *X, int len, int dir, int stride, int K)
       if (dir < 0)
       {
          if (stride2)
-            exp_rotation1(X+i*len, len, dir, stride2, s, c);
-         exp_rotation1(X+i*len, len, dir, 1, c, s);
+            exp_rotation1(X+i*len, len, stride2, s, c);
+         exp_rotation1(X+i*len, len, 1, c, s);
       } else {
-         exp_rotation1(X+i*len, len, dir, 1, c, s);
+         exp_rotation1(X+i*len, len, 1, c, -s);
          if (stride2)
-            exp_rotation1(X+i*len, len, dir, stride2, s, c);
+            exp_rotation1(X+i*len, len, stride2, s, -c);
       }
    }
    /*if (len>=30)
@@ -129,7 +131,8 @@ static void exp_rotation(celt_norm *X, int len, int dir, int stride, int K)
 
 /** Takes the pitch vector and the decoded residual vector, computes the gain
     that will give ||p+g*y||=1 and mixes the residual with the pitch. */
-static void normalise_residual(int * restrict iy, celt_norm * restrict X, int N, int K, celt_word32 Ryy)
+static void normalise_residual(int * restrict iy, celt_norm * restrict X,
+      int N, int K, celt_word32 Ryy, celt_word16 gain)
 {
    int i;
 #ifdef FIXED_POINT
@@ -142,7 +145,7 @@ static void normalise_residual(int * restrict iy, celt_norm * restrict X, int N,
    k = celt_ilog2(Ryy)>>1;
 #endif
    t = VSHR32(Ryy, (k-7)<<1);
-   g = celt_rsqrt_norm(t);
+   g = MULT16_16_P15(celt_rsqrt_norm(t),gain);
 
    i=0;
    do
@@ -150,49 +153,29 @@ static void normalise_residual(int * restrict iy, celt_norm * restrict X, int N,
    while (++i < N);
 }
 
-void alg_quant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, int resynth, ec_enc *enc)
+void alg_quant(celt_norm *X, int N, int K, int spread, int B, celt_norm *lowband,
+      int resynth, ec_enc *enc, celt_int32 *seed, celt_word16 gain)
 {
    VARDECL(celt_norm, y);
    VARDECL(int, iy);
    VARDECL(celt_word16, signx);
-   int j, is;
+   int i, j;
    celt_word16 s;
    int pulsesLeft;
    celt_word32 sum;
-   celt_word32 xy, yy;
-   int N_1; /* Inverse of N, in Q14 format (even for float) */
-#ifdef FIXED_POINT
-   int yshift;
-#endif
+   celt_word32 xy;
+   celt_word16 yy;
    SAVE_STACK;
 
-   if (K==0)
-   {
-      if (lowband != NULL && resynth)
-      {
-         for (j=0;j<N;j++)
-            X[j] = lowband[j];
-         renormalise_vector(X, Q15ONE, N, 1);
-      } else {
-         /* This is important for encoding the side in stereo mode */
-         for (j=0;j<N;j++)
-            X[j] = 0;
-      }
-      return;
-   }
-   K = get_pulses(K);
-#ifdef FIXED_POINT
-   yshift = 13-celt_ilog2(K);
-#endif
+   celt_assert2(K!=0, "alg_quant() needs at least one pulse");
 
    ALLOC(y, N, celt_norm);
    ALLOC(iy, N, int);
    ALLOC(signx, N, celt_word16);
-   N_1 = 512/N;
    
-   if (spread)
-      exp_rotation(X, N, 1, spread, K);
+   exp_rotation(X, N, 1, B, K, spread);
 
+   /* Get rid of the sign */
    sum = 0;
    j=0; do {
       if (X[j]>0)
@@ -217,6 +200,7 @@ void alg_quant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, int r
          sum += X[j];
       }  while (++j<N);
 
+      /* If X is too small, just replace it with a pulse at 0 */
 #ifdef FIXED_POINT
       if (sum <= K)
 #else
@@ -236,9 +220,9 @@ void alg_quant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, int r
          /* It's really important to round *towards zero* here */
          iy[j] = MULT16_16_Q15(X[j],rcp);
 #else
-         iy[j] = floor(rcp*X[j]);
+         iy[j] = (int)floor(rcp*X[j]);
 #endif
-         y[j] = SHL16(iy[j],yshift);
+         y[j] = iy[j];
          yy = MAC16_16(yy, y[j],y[j]);
          xy = MAC16_16(xy, X[j],y[j]);
          y[j] *= 2;
@@ -247,46 +231,49 @@ void alg_quant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, int r
    }
    celt_assert2(pulsesLeft>=1, "Allocated too many pulses in the quick pass");
 
-   while (pulsesLeft > 0)
+   /* This should never happen, but just in case it does (e.g. on silence)
+      we fill the first bin with pulses. */
+#ifdef FIXED_POINT_DEBUG
+   celt_assert2(pulsesLeft<=N+3, "Not enough pulses in the quick pass");
+#endif
+   if (pulsesLeft > N+3)
+   {
+      celt_word16 tmp = pulsesLeft;
+      yy = MAC16_16(yy, tmp, tmp);
+      yy = MAC16_16(yy, tmp, y[0]);
+      iy[0] += pulsesLeft;
+      pulsesLeft=0;
+   }
+
+   s = 1;
+   for (i=0;i<pulsesLeft;i++)
    {
-      int pulsesAtOnce=1;
       int best_id;
-      celt_word16 magnitude;
       celt_word32 best_num = -VERY_LARGE16;
       celt_word16 best_den = 0;
 #ifdef FIXED_POINT
       int rshift;
 #endif
-      /* Decide on how many pulses to find at once */
-      pulsesAtOnce = (pulsesLeft*N_1)>>9; /* pulsesLeft/N */
-      if (pulsesAtOnce<1)
-         pulsesAtOnce = 1;
 #ifdef FIXED_POINT
-      rshift = yshift+1+celt_ilog2(K-pulsesLeft+pulsesAtOnce);
+      rshift = 1+celt_ilog2(K-pulsesLeft+i+1);
 #endif
-      magnitude = SHL16(pulsesAtOnce, yshift);
-
       best_id = 0;
       /* The squared magnitude term gets added anyway, so we might as well 
          add it outside the loop */
-      yy = MAC16_16(yy, magnitude,magnitude);
-      /* Choose between fast and accurate strategy depending on where we are in the search */
-         /* This should ensure that anything we can process will have a better score */
+      yy = ADD32(yy, 1);
       j=0;
       do {
          celt_word16 Rxy, Ryy;
-         /* Select sign based on X[j] alone */
-         s = magnitude;
          /* Temporary sums of the new pulse(s) */
-         Rxy = EXTRACT16(SHR32(MAC16_16(xy, s,X[j]),rshift));
+         Rxy = EXTRACT16(SHR32(ADD32(xy, EXTEND32(X[j])),rshift));
          /* We're multiplying y[j] by two so we don't have to do it here */
-         Ryy = EXTRACT16(SHR32(MAC16_16(yy, s,y[j]),rshift));
-            
-            /* Approximate score: we maximise Rxy/sqrt(Ryy) (we're guaranteed that 
-         Rxy is positive because the sign is pre-computed) */
+         Ryy = ADD16(yy, y[j]);
+
+         /* Approximate score: we maximise Rxy/sqrt(Ryy) (we're guaranteed that
+            Rxy is positive because the sign is pre-computed) */
          Rxy = MULT16_16_Q15(Rxy,Rxy);
-            /* The idea is to check for num/den >= best_num/best_den, but that way
-         we can do it without any division */
+         /* The idea is to check for num/den >= best_num/best_den, but that way
+            we can do it without any division */
          /* OPT: Make sure to use conditional moves here */
          if (MULT16_16(best_den, Rxy) > MULT16_16(Ryy, best_num))
          {
@@ -296,21 +283,18 @@ void alg_quant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, int r
          }
       } while (++j<N);
       
-      j = best_id;
-      is = pulsesAtOnce;
-      s = SHL16(is, yshift);
-
       /* Updating the sums of the new pulse(s) */
-      xy = xy + MULT16_16(s,X[j]);
+      xy = ADD32(xy, EXTEND32(X[best_id]));
       /* We're multiplying y[j] by two so we don't have to do it here */
-      yy = yy + MULT16_16(s,y[j]);
+      yy = ADD16(yy, y[best_id]);
 
       /* Only now that we've made the final choice, update y/iy */
       /* Multiplying y[j] by 2 so we don't have to do it everywhere else */
-      y[j] += 2*s;
-      iy[j] += is;
-      pulsesLeft -= pulsesAtOnce;
+      y[best_id] += 2*s;
+      iy[best_id]++;
    }
+
+   /* Put the original sign back */
    j=0;
    do {
       X[j] = MULT16_16(signx[j],X[j]);
@@ -319,13 +303,10 @@ void alg_quant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, int r
    } while (++j<N);
    encode_pulses(iy, N, K, enc);
    
-   /* Recompute the gain in one pass to reduce the encoder-decoder mismatch
-   due to the recursive computation used in quantisation. */
    if (resynth)
    {
-      normalise_residual(iy, X, N, K, EXTRACT16(SHR32(yy,2*yshift)));
-      if (spread)
-         exp_rotation(X, N, -1, spread, K);
+      normalise_residual(iy, X, N, K, yy, gain);
+      exp_rotation(X, N, -1, B, K, spread);
    }
    RESTORE_STACK;
 }
@@ -333,27 +314,15 @@ void alg_quant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, int r
 
 /** Decode pulse vector and combine the result with the pitch vector to produce
     the final normalised signal in the current band. */
-void alg_unquant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, ec_dec *dec)
+void alg_unquant(celt_norm *X, int N, int K, int spread, int B,
+      celt_norm *lowband, ec_dec *dec, celt_int32 *seed, celt_word16 gain)
 {
    int i;
    celt_word32 Ryy;
    VARDECL(int, iy);
    SAVE_STACK;
-   if (K==0)
-   {
-      if (lowband != NULL)
-      {
-         for (i=0;i<N;i++)
-            X[i] = lowband[i];
-         renormalise_vector(X, Q15ONE, N, 1);
-      } else {
-         /* This is important for encoding the side in stereo mode */
-         for (i=0;i<N;i++)
-            X[i] = 0;
-      }
-      return;
-   }
-   K = get_pulses(K);
+
+   celt_assert2(K!=0, "alg_unquant() needs at least one pulse");
    ALLOC(iy, N, int);
    decode_pulses(iy, N, K, dec);
    Ryy = 0;
@@ -361,13 +330,12 @@ void alg_unquant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, ec_
    do {
       Ryy = MAC16_16(Ryy, iy[i], iy[i]);
    } while (++i < N);
-   normalise_residual(iy, X, N, K, Ryy);
-   if (spread)
-      exp_rotation(X, N, -1, spread, K);
+   normalise_residual(iy, X, N, K, Ryy, gain);
+   exp_rotation(X, N, -1, B, K, spread);
    RESTORE_STACK;
 }
 
-celt_word16 renormalise_vector(celt_norm *X, celt_word16 value, int N, int stride)
+void renormalise_vector(celt_norm *X, int N, celt_word16 gain)
 {
    int i;
 #ifdef FIXED_POINT
@@ -380,20 +348,59 @@ celt_word16 renormalise_vector(celt_norm *X, celt_word16 value, int N, int strid
    for (i=0;i<N;i++)
    {
       E = MAC16_16(E, *xptr, *xptr);
-      xptr += stride;
+      xptr++;
    }
 #ifdef FIXED_POINT
    k = celt_ilog2(E)>>1;
 #endif
    t = VSHR32(E, (k-7)<<1);
-   g = MULT16_16_Q15(value, celt_rsqrt_norm(t));
+   g = MULT16_16_P15(celt_rsqrt_norm(t),gain);
 
    xptr = X;
    for (i=0;i<N;i++)
    {
       *xptr = EXTRACT16(PSHR32(MULT16_16(g, *xptr), k+1));
-      xptr += stride;
+      xptr++;
    }
-   return celt_sqrt(E);
+   /*return celt_sqrt(E);*/
 }
 
+int stereo_itheta(celt_norm *X, celt_norm *Y, int stereo, int N)
+{
+   int i;
+   int itheta;
+   celt_word16 mid, side;
+   celt_word32 Emid, Eside;
+
+   Emid = Eside = EPSILON;
+   if (stereo)
+   {
+      for (i=0;i<N;i++)
+      {
+         celt_norm m, s;
+         m = ADD16(SHR16(X[i],1),SHR16(Y[i],1));
+         s = SUB16(SHR16(X[i],1),SHR16(Y[i],1));
+         Emid = MAC16_16(Emid, m, m);
+         Eside = MAC16_16(Eside, s, s);
+      }
+   } else {
+      for (i=0;i<N;i++)
+      {
+         celt_norm m, s;
+         m = X[i];
+         s = Y[i];
+         Emid = MAC16_16(Emid, m, m);
+         Eside = MAC16_16(Eside, s, s);
+      }
+   }
+   mid = celt_sqrt(Emid);
+   side = celt_sqrt(Eside);
+#ifdef FIXED_POINT
+   /* 0.63662 = 2/pi */
+   itheta = MULT16_16_Q15(QCONST16(0.63662f,15),celt_atan2p(side, mid));
+#else
+   itheta = (int)floor(.5f+16384*0.63662f*atan2(side,mid));
+#endif
+
+   return itheta;
+}
