@@ -2,12 +2,19 @@
 #include "config.h"
 #endif
 
+#define SKIP_CONFIG_H
+
+#ifdef STATIC_MODES
+#undef STATIC_MODES
+#endif
+
 #include <stdio.h>
 #include "kiss_fft.h"
 
 #define CELT_C 
 #include "../libcelt/stack_alloc.h"
 #include "../libcelt/kiss_fft.c"
+#include "../libcelt/mathops.c"
 #include "../libcelt/entcode.c"
 
 
@@ -67,7 +74,7 @@ void test1d(int nfft,int isinverse)
 
     kiss_fft_cpx  * in = (kiss_fft_cpx*)malloc(buflen);
     kiss_fft_cpx  * out= (kiss_fft_cpx*)malloc(buflen);
-    kiss_fft_cfg  cfg = kiss_fft_alloc(nfft,0,0);
+    kiss_fft_state *cfg = kiss_fft_alloc(nfft,0,0);
     int k;
 
     for (k=0;k<nfft;++k) {
