@@ -15,6 +15,14 @@
 #include "../libcelt/entcode.c"
 #include "../libcelt/laplace.c"
 
+#define DATA_SIZE 40000
+
+int ec_laplace_get_start_freq(int decay)
+{
+   celt_uint32 ft = 32768 - LAPLACE_MINP*(2*LAPLACE_NMIN+1);
+   int fs = (ft*(16384-decay))/(16384+decay);
+   return fs+LAPLACE_MINP;
+}
 
 int main(void)
 {
@@ -23,9 +31,12 @@ int main(void)
    ec_enc enc;
    ec_dec dec;
    ec_byte_buffer buf;
+   unsigned char *ptr;
    int val[10000], decay[10000];
    ALLOC_STACK;
-   ec_byte_writeinit(&buf);
+   ptr = malloc(DATA_SIZE);
+   ec_byte_writeinit_buffer(&buf, ptr, DATA_SIZE);
+   //ec_byte_writeinit(&buf);
    ec_enc_init(&enc,&buf);
    
    val[0] = 3; decay[0] = 6000;
@@ -37,8 +48,9 @@ int main(void)
       decay[i] = rand()%11000+5000;
    }
    for (i=0;i<10000;i++)
-      ec_laplace_encode(&enc, &val[i], decay[i]);      
-      
+      ec_laplace_encode(&enc, &val[i],
+            ec_laplace_get_start_freq(decay[i]), decay[i]);
+
    ec_enc_done(&enc);
 
    ec_byte_readinit(&buf,ec_byte_get_buffer(&buf),ec_byte_bytes(&buf));
@@ -46,7 +58,8 @@ int main(void)
 
    for (i=0;i<10000;i++)
    {
-      int d = ec_laplace_decode(&dec, decay[i]);
+      int d = ec_laplace_decode(&dec,
+            ec_laplace_get_start_freq(decay[i]), decay[i]);
       if (d != val[i])
       {
          fprintf (stderr, "Got %d instead of %d\n", d, val[i]);
@@ -54,6 +67,5 @@ int main(void)
       }
    }
    
-   ec_byte_writeclear(&buf);
    return ret;
 }
