@@ -2,6 +2,12 @@
 #include "config.h"
 #endif
 
+#define SKIP_CONFIG_H
+
+#ifdef STATIC_MODES
+#undef STATIC_MODES
+#endif
+
 #include <stdio.h>
 #include "mdct.h"
 #define CELT_C 
@@ -9,6 +15,8 @@
 
 #include "../libcelt/kiss_fft.c"
 #include "../libcelt/mdct.c"
+#include "../libcelt/mathops.c"
+#include "../libcelt/entcode.c"
 
 #ifndef M_PI
 #define M_PI 3.141592653
